@@ -46,10 +46,23 @@ data InfoProxy (c :: Meta) (f :: Type -> Type) (x :: Type) = InfoProxy
 
 type family ToInfo (a :: Type -> Type) :: SOP.T.DatatypeInfo
 type instance ToInfo (M1 D (MetaData n m p False) a) =
-  SOP.T.ADT m n (ToSumInfo a '[])
+  SOP.T.ADT m n (ToSumInfo a '[]) (ToStrictnessInfoss a '[])
 type instance ToInfo (M1 D (MetaData n m p True) a) =
   SOP.T.Newtype m n (ToSingleConstructorInfo a)
 
+type family ToStrictnessInfoss (a :: Type -> Type) (xss :: [[SOP.T.StrictnessInfo]]) :: [[SOP.T.StrictnessInfo]]
+type instance ToStrictnessInfoss (a :+: b)  xss = ToStrictnessInfoss a (ToStrictnessInfoss b xss)
+type instance ToStrictnessInfoss V1         xss = xss
+type instance ToStrictnessInfoss (M1 C _ a) xss = ToStrictnessInfos a '[] ': xss
+
+type family ToStrictnessInfos (a :: Type -> Type) (xs :: [SOP.T.StrictnessInfo]) :: [SOP.T.StrictnessInfo]
+type instance ToStrictnessInfos (a :*: b)  xs = ToStrictnessInfos a (ToStrictnessInfos b xs)
+type instance ToStrictnessInfos U1         xs = xs
+type instance ToStrictnessInfos (M1 S s a) xs = ToStrictnessInfo s ': xs
+
+type family ToStrictnessInfo (s :: Meta) :: SOP.T.StrictnessInfo
+type instance ToStrictnessInfo (MetaSel _ su ss ds) = 'SOP.T.StrictnessInfo su ss ds
+
 type family ToSumInfo (a :: Type -> Type) (xs :: [SOP.T.ConstructorInfo]) :: [SOP.T.ConstructorInfo]
 type instance ToSumInfo (a :+: b)  xs = ToSumInfo a (ToSumInfo b xs)
 type instance ToSumInfo V1         xs = xs
@@ -124,7 +137,7 @@ instance GProductTo U1 where
 
 -- This can most certainly be simplified
 class GSumFrom (a :: Type -> Type) where
-  gSumFrom :: a x -> SOP I xss -> SOP I (ToSumCode a xss)
+  gSumFrom :: a x -> proxy xss -> SOP I (ToSumCode a xss)
   gSumSkip :: proxy a -> SOP I xss -> SOP I (ToSumCode a xss)
 
 instance GSumFrom V1 where
@@ -132,7 +145,10 @@ instance GSumFrom V1 where
   gSumSkip _ xss = xss
 
 instance (GSumFrom a, GSumFrom b) => GSumFrom (a :+: b) where
-  gSumFrom (L1 a) xss = gSumFrom a (gSumSkip (Proxy :: Proxy b) xss)
+  gSumFrom (L1 a) xss = gSumFrom a (toSumCodeProxy xss) where
+    toSumCodeProxy :: proxy xss -> Proxy (ToSumCode b xss)
+    toSumCodeProxy _ = Proxy
+
   gSumFrom (R1 b) xss = gSumSkip (Proxy :: Proxy a) (gSumFrom b xss)
 
   gSumSkip _ xss = gSumSkip (Proxy :: Proxy a) (gSumSkip (Proxy :: Proxy b) xss)
@@ -195,7 +211,7 @@ type GDatatypeInfoOf (a :: Type) = ToInfo (GHC.Rep a)
 -- For more info, see 'Generics.SOP.Generic'.
 --
 gfrom :: (GFrom a, GHC.Generic a) => a -> SOP I (GCode a)
-gfrom x = gSumFrom (GHC.from x) (error "gfrom: internal error" :: SOP.SOP SOP.I '[])
+gfrom x = gSumFrom (GHC.from x) (Proxy :: Proxy '[])
 
 -- | An automatically computed version of 'Generics.SOP.to'.
 --
